from fastapi import FastAPI, HTTPException
from fastapi.middleware.cors import CORSMiddleware
from loguru import logger
import uvicorn
import sys
import json
import argparse
from typing import Dict, Any

from app.core.config import Settings
from app.services.confluence import ConfluenceService
from app.services.chromadb import ChromaDBService
from app.services.rag import RAGService
from app.api.mcp.router import router as mcp_router

# Initialize FastAPI app
app = FastAPI(
    title="Confluence RAG MCP Server",
    description="Model Context Protocol server for Confluence RAG pipeline",
    version="1.0.0"
)

# Add CORS middleware
app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

# Load settings
settings = Settings()

# Configure logging
logger.remove()
logger.add(sys.stderr, format="{time} {level} {message}", level="INFO")
logger.add("logs/app.log", rotation="100 MB")

# Include MCP router
app.include_router(mcp_router, prefix="/mcp", tags=["MCP"])

@app.on_event("startup")
async def startup_event():
    """Initialize services and data on startup"""
    try:
        # Initialize services
        confluence_service = ConfluenceService(settings)
        chromadb_service = ChromaDBService(settings)
        rag_service = RAGService(chromadb_service)
        
        # Store services in app state
        app.state.confluence = confluence_service
        app.state.chromadb = chromadb_service
        app.state.rag = rag_service
        
        # Initial crawl if configured
        if settings.INITIAL_CRAWL:
            logger.info("Starting initial Confluence crawl...")
            documents = await confluence_service.crawl()
            await rag_service.ingest_documents(documents)
            logger.info(f"Initial crawl complete. Ingested {len(documents)} documents.")
            
    except Exception as e:
        logger.error(f"Error during startup: {str(e)}")
        raise

@app.post("/crawl")
async def crawl():
    """Manually trigger Confluence crawl"""
    try:
        documents = await app.state.confluence.crawl()
        await app.state.rag.ingest_documents(documents)
        return {"message": f"Successfully crawled and ingested {len(documents)} documents"}
    except Exception as e:
        logger.error(f"Error during crawl: {str(e)}")
        raise HTTPException(status_code=500, detail=str(e))

@app.get("/health")
async def health():
    """Health check endpoint"""
    return {"status": "healthy"}

def main():
    """Main entry point with CLI argument support"""
    parser = argparse.ArgumentParser(
        description="Confluence RAG MCP Server",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="""
Examples:
  confluence-scraper-mcp                    # Run as MCP stdio server  
  confluence-scraper-mcp --web              # Run as web server on port 8000
  confluence-scraper-mcp --version          # Show version information
        """
    )
    
    parser.add_argument(
        "--web", 
        action="store_true", 
        help="Run as web server instead of MCP stdio server"
    )
    
    parser.add_argument(
        "--port", 
        type=int, 
        default=8000, 
        help="Port for web server (default: 8000)"
    )
    
    parser.add_argument(
        "--host", 
        type=str, 
        default="0.0.0.0", 
        help="Host for web server (default: 0.0.0.0)"
    )
    
    parser.add_argument(
        "--version", 
        action="version", 
        version="confluence-scraper-mcp 0.1.3"
    )
    
    # Parse arguments
    args = parser.parse_args()
    
    if args.web:
        # Run as web server
        logger.info(f"Starting Confluence RAG web server on {args.host}:{args.port}")
        uvicorn.run("app.main:app", host=args.host, port=args.port, reload=True)
    else:
        # Run as MCP stdio server
        logger.info("Starting Confluence RAG MCP stdio server")
        run_mcp_server()

def run_mcp_server():
    """Run the MCP server that processes stdio JSON requests"""
    for line in sys.stdin:
        try:
            request = json.loads(line)
            response = None
            
            if request["type"] == "request":
                if "query" in request["content"]:
                    response = app.state.rag.process_query(request["content"]["query"])
                elif request["content"].get("command") == "crawl":
                    response = app.state.confluence.crawl()
            
            if response:
                print(json.dumps({
                    "version": "1.0",
                    "type": "response",
                    "id": request["id"],
                    "content": response
                }))
                sys.stdout.flush()
            
        except Exception as e:
            logger.error(f"Error processing request: {str(e)}")
            error_response = {
                "version": "1.0",
                "type": "error",
                "id": request.get("id", "unknown"),
                "error": str(e)
            }
            print(json.dumps(error_response))
            sys.stdout.flush()

if __name__ == "__main__":
    main()
