# GitHub Copilot MCP Integration Guide

## Overview

This guide shows you how to integrate the Confluence RAG MCP server with GitHub Copilot in VS Code to provide your AI assistant with access to your Confluence documentation.

## Prerequisites

1. **VS Code** with GitHub Copilot extension installed
2. **Confluence RAG MCP server** installed: `pip install confluence-scraper-mcp`
3. **Confluence API token** and access to your Confluence instance

## Step 1: Install the MCP Server

```bash
# Install the package
pip install confluence-scraper-mcp

# Verify installation
confluence-scraper-mcp --help
```

## Step 2: Create Environment Configuration

Create a `.env` file in your project directory:

```bash
# Required Confluence settings
CONFLUENCE_BASE_URL=https://your-domain.atlassian.net
CONFLUENCE_TOKEN=your-api-token
CONFLUENCE_SPACE_KEY=your-default-space

# Optional settings
CHROMA_PERSIST_DIR=data/chroma
CHROMA_COLLECTION_NAME=confluence_docs
EMBEDDING_MODEL=sentence-transformers/all-MiniLM-L6-v2
CHUNK_SIZE=512
CHUNK_OVERLAP=50
TOP_K=3
SIMILARITY_THRESHOLD=0.7
```

## Step 3: Configure GitHub Copilot

You have two options for configuring the MCP server with GitHub Copilot:

### Method 1: VS Code Settings UI (Direct Configuration)

1. Open VS Code Settings (`Cmd+,` on Mac, `Ctrl+,` on Windows/Linux)
2. Search for "copilot"
3. Look for "GitHub Copilot: MCP Servers" setting
4. Click "Edit in settings.json"
5. Add the MCP server configuration:

```json
{
  "github.copilot.chat.mcpServers": {
    "confluence-rag": {
      "command": "confluence-scraper-mcp",
      "args": [],
      "env": {
        "CONFLUENCE_BASE_URL": "https://your-domain.atlassian.net",
        "CONFLUENCE_TOKEN": "your-api-token",
        "CONFLUENCE_SPACE_KEY": "your-space-key"
      }
    }
  }
}
```

### Method 2: MCP Configuration File (Recommended)

**Step 2a: Create MCP Configuration File**

Create a file called `mcp-config.json` in your project or home directory:

```json
{
  "mcpServers": {
    "confluence-rag": {
      "command": "confluence-scraper-mcp",
      "args": [],
      "env": {
        "CONFLUENCE_BASE_URL": "https://your-domain.atlassian.net",
        "CONFLUENCE_TOKEN": "your-api-token",
        "CONFLUENCE_SPACE_KEY": "your-space-key"
      }
    }
  }
}
```

**Step 2b: Configure VS Code to Use MCP File**

Add this to your VS Code `settings.json`:

```json
{
  "github.copilot.chat.mcpConfigPath": "./mcp-config.json"
}
```

Or for a global configuration:

```json
{
  "github.copilot.chat.mcpConfigPath": "~/.config/mcp/config.json"
}
```

### Method 3: Direct settings.json Edit

1. Open Command Palette (`Cmd+Shift+P` / `Ctrl+Shift+P`)
2. Type "Preferences: Open User Settings (JSON)"
3. Add the MCP configuration to your settings:

```json
{
  "github.copilot.chat.mcpServers": {
    "confluence-rag": {
      "command": "confluence-scraper-mcp",
      "args": [],
      "env": {
        "CONFLUENCE_BASE_URL": "https://your-domain.atlassian.net",
        "CONFLUENCE_TOKEN": "your-api-token",
        "CONFLUENCE_SPACE_KEY": "your-space-key"
      }
    }
  }
}
```

## Step 4: Advanced Configuration (Optional)

For more advanced setups, you can configure multiple Confluence spaces with different settings.

### Option A: Multiple Servers in settings.json

```json
{
  "github.copilot.chat.mcpServers": {
    "confluence-api-docs": {
      "command": "confluence-scraper-mcp",
      "args": [],
      "env": {
        "CONFLUENCE_BASE_URL": "https://your-domain.atlassian.net",
        "CONFLUENCE_TOKEN": "your-api-token",
        "CONFLUENCE_SPACE_KEY": "API",
        "CHROMA_COLLECTION_NAME": "api_docs"
      }
    },
    "confluence-architecture": {
      "command": "confluence-scraper-mcp",
      "args": [],
      "env": {
        "CONFLUENCE_BASE_URL": "https://your-domain.atlassian.net",
        "CONFLUENCE_TOKEN": "your-api-token",
        "CONFLUENCE_SPACE_KEY": "ARCH",
        "CHROMA_COLLECTION_NAME": "architecture_docs"
      }
    }
  }
}
```

### Option B: MCP Configuration File (Recommended)

Use the provided `examples/mcp-config-advanced.json` file and reference it in VS Code settings:

```json
{
  "github.copilot.chat.mcpConfigPath": "./examples/mcp-config-advanced.json"
}
```

This approach provides:
- **Better organization**: Separate MCP config from VS Code settings
- **Version control**: Track MCP configuration changes
- **Portability**: Share configurations across teams
- **Flexibility**: Different configurations for different projects

## Step 5: Test the Configuration

1. **Restart VS Code** after making configuration changes
2. **Open GitHub Copilot Chat** (`Cmd+I` / `Ctrl+I`)
3. **Test with a query** about your Confluence content:
   ```
   Tell me about our API authentication methods
   ```
4. **Check if Copilot** mentions using Confluence documentation in its response

## Step 6: Initial Setup and Crawling

Before using the MCP server, you need to crawl your Confluence content:

```bash
# Start the server in background
confluence-scraper-mcp --web &

# Trigger initial crawl
curl -X POST http://localhost:8000/crawl

# Check health
curl http://localhost:8000/health
```

## Troubleshooting

### 1. MCP Server Not Found
- Ensure `confluence-scraper-mcp` is installed and available in PATH
- Try using full path: `/usr/local/bin/confluence-scraper-mcp` or similar

### 2. Authentication Issues
- Verify your Confluence token has proper permissions
- Test the token with a direct API call:
  ```bash
  curl -H "Authorization: Bearer your-token" \
       "https://your-domain.atlassian.net/rest/api/content"
  ```

### 3. No Context in Responses
- Ensure you've run the initial crawl
- Check that the ChromaDB data directory exists and has content
- Verify the space key is correct

### 4. VS Code Configuration Issues
- Restart VS Code after configuration changes
- Check VS Code Developer Console for errors (`Help > Toggle Developer Tools`)
- Verify JSON syntax in settings.json
- **For MCP config files**: Ensure the file path is correct and accessible
- **Tip**: Use MCP config files for better error isolation and debugging

### 5. Configuration Management
- **Use MCP config files** for team sharing and version control
- **Validate JSON syntax** before committing configuration files
- **Test with simple configs first**, then add complexity

## Usage Examples

Once configured, you can ask Copilot questions like:

- "How do we handle user authentication in our system?"
- "What are the deployment procedures for our microservices?"
- "Show me the API documentation for the user service"
- "What are our coding standards and best practices?"

Copilot will automatically search your Confluence documentation and include relevant context in its responses.

## Advanced Features

### Custom Prompts
You can create custom prompts that specifically reference documentation:

```
@confluence search for "deployment pipeline" and explain how to deploy to production
```

### Space-Specific Queries
If you have multiple spaces configured:

```
@confluence-api-docs explain the authentication flow
@confluence-architecture show me the system architecture
```

## Maintenance

### Regular Updates
- Re-crawl Confluence periodically to keep content fresh:
  ```bash
  curl -X POST http://localhost:8000/crawl
  ```

### Monitor Performance
- Check logs for any issues:
  ```bash
  tail -f logs/app.log
  ```

### Update Configuration
- Modify space keys or add new spaces as needed
- Restart VS Code after configuration changes

## Security Notes

1. **API Token Security**: Store your Confluence token securely
2. **Network Access**: Ensure the MCP server can access your Confluence instance
3. **Permissions**: Use a token with minimal required permissions
4. **Local Data**: ChromaDB stores embeddings locally - consider backup/cleanup procedures

## Support

If you encounter issues:
1. Check the [GitHub Issues](https://github.com/akhilthomas236/confluence-scraper-mcp/issues)
2. Review the [README](https://github.com/akhilthomas236/confluence-scraper-mcp)
3. Check the [PyPI package](https://pypi.org/project/confluence-scraper-mcp/)
